module Veritas_Infoscale
    module Install
        require 'chef/log'
        require 'fileutils'
        Logger = Chef::Log
        include Chef::Mixin::ShellOut
           
            #Function Name: verify_install_script
            #Inputs: installer_path
            #Output: Boolean value: ret
            #Description: Verifies if the Attribute install_script has valid installer path.
            #             The function returns 1 if the intaller path does not exist and returns
            #             0 on success.

            def verify_install_script(installer_path)
                ret = 0
                if (installer_path.empty?) || !(File.exist?(installer_path))
                    ret = 1
                    Logger.error("Attribute install_script is empty or the " +
                                 "installer script #{installer_path} does not exist")
                end
                return ret
            end

            #Function Name: get_responsefile_path
            #Inputs: responsefile_path, default_path
            #Output: responsefile_path,file_exists
            #Description: This function checks Attribute responsefile_path
            #1) If responsefile_path is empty, the functions defaults to /opt/VRTS/install/chef
            #   if /opt/VRTS/install location is present on the system. If /opt/VRTS/install is 
            #   not present, ENV['HOME'] is used as default location.
            #2) If responsefile_path is given by the user and does not exist on the system,
            #   the get_responsefile_path function creates responsefile_path with 0644 permission.
            #3) If creation of responsefile_path fails, file_exists is set to false and an exception is raised.

            def get_responsefile_path(responsefile_path, default_path)
                veritas_install_path = "/opt/VRTS/install"
                file_exists = true
                if (responsefile_path.empty?)
                    Logger.info('Attribute responsefile_path is empty')
                    Logger.info("Using default responsefile path: #{default_path}")
                    if (default_path.eql?("/opt/VRTS/install/chef")) && !(File.directory?(default_path))
                        if !(File.directory?(veritas_install_path))
                            responsefile_path = ENV['HOME']
                        else
                            if ((Dir.mkdir("/opt/VRTS/install/chef",0644))== 0)#if directory creation is successful
                                responsefile_path = default_path
                            else
                                Logger.info("Failed to create chef directory in #{veritas_install_path}")
                                responsefile_path = ENV['HOME']
                                Logger.info("Using #{ENV['HOME']} as responsefile location")
                            end
                        end
                    else
                        responsefile_path = default_path
                    end
                elsif (!(File.directory?(responsefile_path)))
                      Logger.warn('Path given in responsefile_path Attribute does not exist')
                      if !((FileUtils.mkdir_p(responsefile_path, :mode => 0644)).eql?([responsefile_path]))#if directory creation is successful
                          Logger.error("Unable to create directory responsefile_path #{responsefile_path}")
                          file_exists = false
                      end
                end
                return responsefile_path,file_exists
            end

            #Function Name: move_responsefile
            #Inputs: responsefile_path, responsefile
            #Output: This function will give a warning if responsefile move fails
            #Description: This function is used to move responsefile to /opt/VRTS/install/chef
            #             location if the responsefile due to some reason is created in ENV['HOME'].
            #             In case the function is unable to move the responsefile, it will log an error
            #             message to inform the user and the responsefile will continue to be in 
            #             ENV['HOME'].

            def move_responsefile(responsefile_path, responsefile)
                default_path = "/opt/VRTS/install/chef"
                move = 0
                if File.exist?("#{responsefile_path}/#{responsefile}")
                    if !(File.directory?(default_path))
                         if ((Dir.mkdir("/opt/VRTS/install/chef",0644))== 0) 
                             move = 1
                         else
                             Logger.warn("Unable to create /opt/VRTS/install/chef")
                             Logger.info("Response file will remain in #{responsefile_path}")
                         end
                    else
                        move = 1
                    end
                     
                     if move == 1
                         if !(FileUtils.mv("#{responsefile_path}/#{responsefile}", "#{default_path}/#{responsefile}"))
                             Logger.warn("Unable to move #{responsefile_path}/#{responsefile}")
                         else
                             Logger.info("Successfully moved Response file from " +
                                         "#{responsefile_path} to #{default_path}")
                         end
                     end

                else
                    Logger.warn("Unable to move #{responsefile_path}/#{responsefile} as the file does not exist")
                end
            end

            #Function Name: verify_phase1_order
            #Inputs: phase1_order, system_list
            #Output: Boolean value: 0 on successful verification and 1 in case of failure
            #Description: This function is used to verify Attribute phase1_order.

            def verify_phase1_order(phase1_order, system_list)
                if(phase1_order.is_a?(Integer) || phase1_order.empty?)
                    Logger.warn("Attribute phase1_order is empty or contains invalid value")
                    return 1
                end
                case phase1_order
                when Hash
                    if (phase1_order.keys.count > system_list.count)
                        Logger.warn("The number of subclusters defined in attribute phase1_order" +
                                " exceeds the number of systems")
                    end
                    #check for keys
                    phase1_order.keys.count.times do |keyorder|
                        keyorder = keyorder.to_s
                        if ( !phase1_order.key?(keyorder))
                            Logger.warn("Attribute phase1_order not specified in correct format")
                            Logger.warn("key value: #{keyorder} not specified")
                            return 1
                        end
                    end

                when Array
                    if (phase1_order.count > system_list.count)
                        Logger.warn("The number of systems defined in attribute" + 
                                    " phase1_order exceeds number of systems")
                        return 0
                    end

                when String
                    return 0

                else
                    return 1
                end

                return 0 #successful verification
            end

            #Function Name: IsAttributeBool
            #Inputs: attribute, raise_val, attr_name
            #Output: 1, 0, -1
            #Description: This function is used to validate boolean attributes.
            #1) Returns 1 if attribute is "1" or 1.
            #2) Returns 0 if attribute is "0" or 0.
            #3) If the attribute does not match the above conditions,
            #   raise_val is checked.
            #   i) If raise_val is 1, then an exception is raised.
            #   ii)If raise_val is 0, then -1 is returned.
 
            def IsAttributeBool(attribute, raise_val, attr_name)
                if ( attribute.eql?("0") || attribute.eql?(0) )
                    return 0
                elsif (attribute.eql?("1") || attribute.eql?(1))
                    return 1
                else 
                    if raise_val == 1
                        raise "Attribute #{attr_name} is invalid. " +
                                "Attribute #{attr_name} can only have 0 or 1 as valid values"
                    else
                        Logger.warn("Attribute #{attr_name} is invalid. " +
                                    "Attribute #{attr_name} can only have 0 or 1 as valid values")
                        return -1
                    end
                end
            end
    end
end
